/*
SD / MMC Bootloader for Atmel Atmega microcontrollers

Author:
	Nik Bamert
	nikbamert@bluewin.ch / admin@nikbamert.com
	www.nikbamert.com
	
License:	
	Creative Commons : Attribution-NonCommercial 2.5
	
	You are free:
	
		* to copy, distribute, display, and perform the work
		* to make derivative works
	
	Under the following conditions:
		
	Attribution. You must attribute the work in the manner specified by the author or licensor.
		
	Noncommercial. You may not use this work for commercial purposes.
	
		* For any reuse or distribution, you must make clear to others the license terms of this work.
		* Any of these conditions can be waived if you get permission from the copyright holder.
	
	Your fair use and other rights are in no way affected by the above.
	
	This is a human-readable summary of the Legal Code (the full license). 
	
	http://creativecommons.org/licenses/by-nc/2.5/legalcode
*/

#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>

#include "mmc_lib.h"
#include "fat.h"
#include "bootfunction.h"

//Einen Sektor vom Speichermedium lesen.
//Um die miniFatlib einfach an andere Sd/mmc, harddisk, cf etc. libs anzupassen, einfach
//deren sektorlesefunktion unten einfgen

void FAT_ReadSector(uint32_t sector){
	mmc_start_read_block(sector, 9); // MBR auslesen
	mmc_read_buffer(&buffer[0],512);
	mmc_stop_read_block();

}

/* 
void InitializeFilehandles(void){
	uint8_t cnt;
	Alle Filehandles durchlaufen
	for(cnt=0;cnt<MAX_OPENED_FILES;cnt++){
		files[cnt].startcluster = FILE_CLOSED;
	}
	

}

uint8_t fclose(uint8_t filehandle){
	files[filehandle].startcluster = FILE_CLOSED;
	return filehandle;
}*/




/*Initialisiert die FAT (FAT16/32) (im MOMENT NUR FAT32!!)
###############################################################################################*/
uint8_t fat_init(void)
{
	unsigned int vbr_start;

	mmc_set_blocklen(512);
	FAT_ReadSector(0);
	

	
	vbr_start = buffer[454]; // VBR Adresse der ersten Partition holen(vielleicht noch etwas buggy!)
	//if(buffer[PARTITION_TYPE] == 0x0B || buffer[PARTITION_TYPE] == 0x0C) Filesystem.partitiontype = FAT32;
	//if(buffer[PARTITION_TYPE] == 0x04 || buffer[PARTITION_TYPE] == 0x06 || buffer[PARTITION_TYPE] == 0x0e) Filesystem.partitiontype = FAT16;
	FAT_ReadSector(vbr_start); // VBR auslesen
	
	
	
	Filesystem.fatbegin = vbr_start + (unsigned short)buffer[BPB_RsvdSecCnt]; //Number_of_Reserved_Sectors;
	Filesystem.clusbegin = vbr_start + (unsigned short)buffer[BPB_RsvdSecCnt]  + (2 * *((unsigned long *)&buffer[BPB_SECperFat]));
	Filesystem.secsperclus = (unsigned char)buffer[BPB_SecPerClus];
	

	return(FAT_OK);
}


/*FOPEN FUNCTION--------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------*/

uint8_t fopen(unsigned char *searchfile) 
{	

    uint32_t Cluster,sectoread,Wert;
	unsigned char search[12],minibuf[4],i=0, cnt=0;
    unsigned short EntryOffset;
 

	


	/*Dateiname der gesuchten datei in ein char array einfllen*/
	
		for(i=0;i<11;i++){
			search[i] = *searchfile++;
		}
		search[11]=0x00;i=0;
	/*---------------------------------------------------------*/
		
	Cluster=2; //Cluster auf 2 Setzen(Anfang des Root Directory)
	
	/*Suchfunktion um den ersten Cluster des Files zu finden*/
	while(Cluster){
	
		for(uint8_t i=0;i<Filesystem.secsperclus;i++){
			
			sectoread = Filesystem.clusbegin + ((Cluster - 2) * Filesystem.secsperclus)+i;//Berechnung um den gewnschten Sektor auszurechnen
		  	FAT_ReadSector(sectoread);
			
				for(uint8_t j=0;j<16;j++){ //(16 ist die Anzahl Eintrage pro Sektor)
					EntryOffset=j*32; //Lookup aus dem Array damit nichts gerechnet werden muss   
					if(!(buffer[EntryOffset] == 0xE5 || buffer[EntryOffset] == 0x00)){
					
						for(uint8_t k=0;k<11;k++){
							if(buffer[EntryOffset+ k] == search[k]) {cnt++;}else{break;} //Meine Eigene 'strcmp'
						}
					}
						minibuf[0] = buffer[EntryOffset+26];
						minibuf[1] = buffer[EntryOffset+27];
						minibuf[2] = buffer[EntryOffset+20];
						minibuf[3] = buffer[EntryOffset+21];
						
						files.startcluster =  *((uint32_t *)&minibuf[0]);
						
					
					//	Wert = *((uint32_t *)&buffer[EntryOffset+28]);
					//	files.size =( ( Wert & 0xff000000 ) >> 24 ) | ( ( Wert & 0x000000ff ) << 24 ) | ( ( Wert & 0x00ff0000 ) >> 8 ) | ( ( Wert & 0x0000ff00 ) << 8 );
						
					
						
					if(cnt == 11) return(FILE_FOUND); // Waren 11 Zeichen gleich? (11=lnge des Dateinamens)
					
					if(Cluster == 0) return(FILE_NOT_FOUND);     //Directory durchlaufen aber nichts gefunden->0 zurckgeben
					cnt=0;
				}
		}
		Cluster = followClusterChain(Cluster);
	}
	
	return (FATAL_ERROR);
}





//Fat helper functions 
//---------------------------------------------------------------
uint32_t followClusterChain(unsigned long cluster)
{

	
	//#if USE_FAT32
	FAT_ReadSector(Filesystem.fatbegin + (cluster / 128));
	unsigned short nFatOffset  = ((cluster % 128) * 4); // 0...128 Offset des fat eintrages(4 byte lang) ausrechnen
	//#endif
	
	/*#if USE_FAT16
	FAT_ReadSector(Filesystem.fatbegin + (cluster / 256));
	unsigned short nFatOffset  = ((cluster % 256) * 2); // 0...128 Offset des fat eintrages(4 byte lang) ausrechnen
	#endif*/
	
	return (*((uint32_t*)&buffer[nFatOffset])); // An der Stelle nFatOffset den Fat eintrag auslesen und zurckgeben...

}


uint32_t fprogram(uint8_t filehandle)
{
	uint32_t handle = files.startcluster;
    uint32_t SectorToReadFrom = ((handle-2)*Filesystem.secsperclus)+Filesystem.clusbegin;
	uint32_t page_counter,remaining;
    uint8_t i;
	
	page_counter=0;

	while(1) // solange es was zu lesen gibt, lies ;-)
    {
		
        for(i=0;i<Filesystem.secsperclus;i++){
			FAT_ReadSector(SectorToReadFrom);
		 
			remaining=512;
			while(remaining){
				boot_program_page(page_counter,&buffer[512-remaining]);
				// SPM_PAGESIZE ist im jeweiligen AVRdef File definiert.
				page_counter+=SPM_PAGESIZE; // Pagecounter erhhen
				remaining-=SPM_PAGESIZE; 
			}		
			
		 
			SectorToReadFrom++;
		  }
      
		handle = followClusterChain(handle);
		if(handle==0x0fffffff)return(0); //Fileende? -> return
		SectorToReadFrom = ((handle-2)*Filesystem.secsperclus)+Filesystem.clusbegin;
       
    }
   
    return(1);
}
