/*
SD / MMC Bootloader for Atmel Atmega microcontrollers

Author:
	Nik Bamert
	nikbamert@bluewin.ch / admin@nikbamert.com
	www.nikbamert.com
	
License:	
	Creative Commons : Attribution-NonCommercial 2.5
	
	You are free:
	
		* to copy, distribute, display, and perform the work
		* to make derivative works
	
	Under the following conditions:
		
	Attribution. You must attribute the work in the manner specified by the author or licensor.
		
	Noncommercial. You may not use this work for commercial purposes.
	
		* For any reuse or distribution, you must make clear to others the license terms of this work.
		* Any of these conditions can be waived if you get permission from the copyright holder.
	
	Your fair use and other rights are in no way affected by the above.
	
	This is a human-readable summary of the Legal Code (the full license). 
	
	http://creativecommons.org/licenses/by-nc/2.5/legalcode
*/

#include "config.h"
#include <avr/io.h>
#include <avr/delay.h>
#include "bootfunction.h"
#include "mmc_lib.h"
#include "fat.h"

#define LED_ON 1
#define LED_OFF 0

#if STATUS_LED_SUPPORT
void led_flash(uint8_t flashes){

	LED_DIRECTION |= 1<<LED_PIN;
		
	while(flashes--){
		_delay_ms(50);
		LED_PORT &= ~(1<<LED_PIN);
		_delay_ms(50);
		LED_PORT |= 1<<LED_PIN;
	}

}

void led(uint8_t status){
	LED_DIRECTION |= 1<<LED_PIN;
	
	if(status == LED_ON) LED_PORT |= 1<<LED_PIN;
	if(status == LED_OFF) LED_PORT &= ~(1<<LED_PIN);

}
#endif
	
int main(void){
	uint8_t filehandle;
	uint8_t status;
	

	status = mmc_init();
	if(status == MMC_OK){
		
		//Programmierled einschalten
		#if STATUS_LED_SUPPORT
		led(LED_ON);
		#endif
		//Fat initialisieren
		fat_init();
		
		//Firmwarefile ffnen
		filehandle = fopen("MAIN    BIN");
		
		if(filehandle == FILE_FOUND) {	
			//Neues Programm laden (filehandle bekannt, da nur ein file aufs mal offen sein darf -> handle '0')
			fprogram(FIRMWARE_FILE);
			
		}else{
			//File konnte nicht gefunden werden -> Error
			#if STATUS_LED_SUPPORT
			led_flash(5);
			#endif
		}
	}else{
		//MMC / SD Karte konnte nicht gefunden werden -> Error
		#if STATUS_LED_SUPPORT
		led_flash(5);			
		#endif
	}


	//Programmierled abschalten
	#if STATUS_LED_SUPPORT
	//led_flash(5);
	led(LED_OFF);
	#endif
	
	//Vorbereitungen um an den Programmanfang zu springen.
	//Vielleicht noch bei anderen, neuen atmegas nicht mehr GICR sondern MCUCR!!!
	#if defined (__AVR_ATmega88__) || defined (__AVR_ATmega168__)
		uint8_t my_GICR_IVCE  = MCUCR | _BV(IVCE);
		uint8_t my_GICR_IVSEL = MCUCR & ~_BV(IVSEL);
	
		MCUCR = my_GICR_IVCE;    // diese beiden Schreibbefehle mssen direkt hintereinander erfolgen
		MCUCR = my_GICR_IVSEL;  // (max. 4 Taktzyklen)
	#else
		uint8_t my_GICR_IVCE  = GICR | _BV(IVCE);
		uint8_t my_GICR_IVSEL = GICR & ~_BV(IVSEL);
	
		GICR = my_GICR_IVCE;    // diese beiden Schreibbefehle mssen direkt hintereinander erfolgen
		GICR = my_GICR_IVSEL;  // (max. 4 Taktzyklen)
	
	#endif
	
	//An den Programmanfang springen
	(*((void(*)(void))0x00))(); 
	while(1);
	


}
